﻿
using System;
using System.Linq;
using TenCrowns.GameCore;
using TenCrowns.GameCore.Text;
using static TenCrowns.GameCore.Text.TextExtensions;
using Mohawk.SystemCore;
using UnityEngine;

namespace ExtendedInheritanceScreen
{
    class ExtendedInheritanceScreenCharacter : Character
    {
        public ExtendedInheritanceScreenCharacter() : base()
        {
            return;
        }

        public override void doTurn()
        {
            if (isDead())
            {
                return;
            }

            if (isCouncil())
            {
                increaseXP(infos().council(getCouncil()).miXP);
            }

            if (isCityGovernor())
            {
                increaseXP(cityGovernor().culture().miXP);
            }

            testUpgrade(false);

            {
                bool bValid = true;

                if (isLeader())
                {
                    if (player().findNumHeirs() == 0)
                    {
                        if (getAge() < 80)
                        {
                            bValid = false;
                        }
                    }
                }
                else if (isHeir())
                {
                    if (player().findNumHeirs() == 1)
                    {
                        if (getAge() < 60)
                        {
                            bValid = false;
                        }
                    }
                }

                if (bValid)
                {
                    if (isDoomed())
                    {
                        if (game().randomNext(5) != 0)
                        {
                            makeDead();

                            return;
                        }
                    }

                    foreach (TraitType eLoopTrait in getTraits())
                    {
                        int iProb = infos().utils().range((infos().trait(eLoopTrait).miDieProb + infos().trait(eLoopTrait).maiMortalityDieProb[(int)(game().getMortality())]), 0, 100);
                        if (iProb > 0)
                        {
                            if (game().testRandomPercent(iProb))
                            {
                                if (isLeader() && !isDoomed())
                                {
                                    makeDoomed();

                                    if (hasPlayer())
                                    {
                                        player().pushNextPopupText(TextManager.TEXT("TEXT_GAME_DOOMED_POPUP"));
                                    }
                                }
                                else
                                {
                                    makeDead();

                                    return;
                                }
                            }
                        }
                    }
                }
            }

            {
                bool bTrait = !isDoomed();

                if (bTrait)
                {
                    if (game().getTurn() > 20)
                    {
                        int iDecade = Math.Min((Constants.CHARACTER_DECADES - 1), (getAge() / 10));

                        for (TraitType eLoopTrait = 0; eLoopTrait < infos().traitsNum(); eLoopTrait++)
                        {
                            if (!isTrait(eLoopTrait))
                            {
                                int iProb = infos().trait(eLoopTrait).maiDecadeProb[iDecade];
                                if (iProb > 0)
                                {
                                    bool bValid = true;

                                    foreach (TraitType eOtherTrait in getTraits())
                                    {
                                        if (infos().trait(eLoopTrait).mabSkipDecade[(int)eOtherTrait])
                                        {
                                            bValid = false;
                                            break;
                                        }
                                    }

                                    if (bValid)
                                    {
                                        if (game().testRandomPercent(iProb))
                                        {
                                            addTrait(eLoopTrait);

                                            if (isLeaderOrSpouseOrHeirOrHeirSpouseOrSuccessor())
                                            {
                                                player().addTurnSummary(TextManager.TEXT("TEXT_GAME_CHARACTER_GAINS_TRAIT", HelpText.buildCharacterLinkVariable(this, true, player()), HelpText.buildTraitLinkVariable(eLoopTrait)));
                                            }

                                            bTrait = false;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (bTrait)
                {
                    using (var traitListScoped = CollectionCache.GetListScoped<TraitType>())
                    {
                        getTraits(traitListScoped.Value);

                        foreach (TraitType eLoopTrait in traitListScoped.Value)
                        {
                            for (TraitType eOtherTrait = 0; eOtherTrait < infos().traitsNum(); eOtherTrait++)
                            {
                                int iProb = infos().trait(eLoopTrait).maiTraitProb[(int)eOtherTrait];
                                if (iProb > 0)
                                {
                                    if (game().testRandomPercent(iProb))
                                    {
                                        removeTrait(eLoopTrait);
                                        addTrait(eOtherTrait);

                                        if (isLeaderOrSpouseOrHeirOrHeirSpouseOrSuccessor())
                                        {
                                            player().addTurnSummary(TextManager.TEXT("TEXT_GAME_CHARACTER_GAINS_TRAIT", HelpText.buildCharacterLinkVariable(this, true, player()), HelpText.buildTraitLinkVariable(eOtherTrait)));
                                        }

                                        bTrait = false;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }

                if (bTrait)
                {
                    using (var traitListScoped = CollectionCache.GetListScoped<TraitType>())
                    {
                        getTraits(traitListScoped.Value);

                        foreach (TraitType eLoopTrait in traitListScoped.Value)
                        {
                            int iProb = infos().trait(eLoopTrait).miRemoveProb;
                            if (iProb > 0)
                            {
                                if (game().testRandomPercent(iProb))
                                {
                                    removeTrait(eLoopTrait);

                                    if (isLeaderOrSpouseOrHeirOrHeirSpouseOrSuccessor())
                                    {
                                        player().addTurnSummary(TextManager.TEXT("TEXT_GAME_CHARACTER_LOSES_TRAIT", HelpText.buildCharacterLinkVariable(this, true, player()), HelpText.buildTraitLinkVariable(eLoopTrait)));
                                    }

                                    bTrait = false;
                                    break;
                                }
                            }
                        }
                    }
                }
            }

            if (isUnitGeneral() && !canRemainGeneral())
            {
                Unit pUnit = unitGeneral();
                pUnit.clearGeneral();

                if (hasPlayer())
                {
                    if (player().isHuman())
                    {
                        ChooseItemDecision pDecisionData = new ChooseItemDecision(TextManager.TEXT("TEXT_GAME_DECISION_RETIRING_GENERAL_TITLE"),
                            TextManager.TEXT("TEXT_GAME_DECISION_RETIRING_GENERAL", HelpText.buildCharacterLinkVariable(this, true, player()), HelpText.buildUnitLinkVariable(pUnit)),
                            pUnit.getTileID(), TextManager.TEXT("TEXT_GAME_DECISION_RETIRING_GENERAL_PROMPT"), ItemType.ADD_GENERAL, pUnit.getID());
                        player().pushDecisionDataNext(pDecisionData);
                    }
                }
            }

            if (isMale() && isRoyal() && hasPlayer())
            {
                if (!isRetired() && canHaveChildren() && !isLeader())
                {
                    if (game().getTurn() > 10)
                    {
                        if (game().random().Next(10) != 0)
                        {
                            int iProb = 0;

                            int iLivingChildren = countLivingChildren();

                            if (isLeader() && !(player().hasHeir()))
                            {
                                iProb += 30;
                            }
                            else if (iLivingChildren == 0)
                            {
                                iProb += 20;
                            }
                            else if (iLivingChildren == 1)
                            {
                                iProb += 10;
                            }
                            else
                            {
                                iProb += (10 / iLivingChildren);
                            }

                            iProb = infos().utils().modify(iProb, getTraitBirthModifier());

                            if (game().testRandomPercent(iProb))
                            {
                                game().giveBirth(null, this);
                            }
                        }
                    }
                }
            }

            if (isMale() && (isRoyal() || isRoyalSpouse()) && hasPlayer())
            {
                if (canHaveChildren())
                {
                    foreach (int iSpouseID in getSpouses())
                    {
                        Character pLoopSpouse = game().character(iSpouseID);

                        if (pLoopSpouse.isAlive())
                        {
                            if (pLoopSpouse.canHaveChildren())
                            {
                                bool bValid = true;

                                if (isRoyal() && isRetired())
                                {
                                    if (!(pLoopSpouse.isRoyal()) || pLoopSpouse.isRetired())
                                    {
                                        bValid = false;
                                    }
                                }
                                else if (pLoopSpouse.isRoyal() && pLoopSpouse.isRetired())
                                {
                                    if (!isRoyal() || isRetired())
                                    {
                                        bValid = false;
                                    }
                                }

                                if (bValid)
                                {
                                    int iProb = 0;

                                    int iLivingChildren = countLivingChildren();

                                    if (isLeaderOrSpouse() && !(player().hasHeir()))
                                    {
                                        iProb += 30;
                                    }
                                    else if (iLivingChildren == 0)
                                    {
                                        iProb += 20;
                                    }
                                    else if (iLivingChildren == 1)
                                    {
                                        iProb += 10;
                                    }
                                    else
                                    {
                                        iProb += (10 / iLivingChildren);
                                    }

                                    iProb = infos().utils().modify(iProb, getTraitBirthModifier());
                                    iProb = infos().utils().modify(iProb, pLoopSpouse.getTraitBirthModifier());

                                    if (isLeader())
                                    {
                                        iProb = infos().utils().modify(iProb, infos().opinionCharacter(player().calculateCharacterOpinion(pLoopSpouse)).miBirthModifier);
                                    }
                                    else if (pLoopSpouse.isLeader())
                                    {
                                        iProb = infos().utils().modify(iProb, infos().opinionCharacter(player().calculateCharacterOpinion(this)).miBirthModifier);
                                    }

                                    if (game().testRandomPercent(iProb))
                                    {
                                        if (isMale())
                                        {
                                            game().giveBirth(pLoopSpouse, this);
                                        }
                                        else
                                        {
                                            game().giveBirth(this, pLoopSpouse);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            // Dale Start
            if (isMale() && !(isRoyal() || isRoyalSpouse()) && hasPlayer())
            {
                if (canHaveChildren())
                {
                    foreach (int iSpouseID in getSpouses())
                    {
                        Character pLoopSpouse = game().character(iSpouseID);

                        if (pLoopSpouse.isAlive())
                        {
                            if (pLoopSpouse.canHaveChildren())
                            {
                                bool bValid = true;

                                if (isRetired() || pLoopSpouse.isRetired())
                                {
                                    bValid = false;
                                }

                                if (bValid)
                                {
                                    int iProb = 0;

                                    int iLivingChildren = countLivingChildren();

                                    if (isFamilyHead() && iLivingChildren == 0)
                                    {
                                        iProb += 30;
                                    }
                                    else if (iLivingChildren == 0)
                                    {
                                        iProb += 20;
                                    }
                                    else if (iLivingChildren == 1)
                                    {
                                        iProb += 10;
                                    }
                                    else
                                    {
                                        iProb += (10 / iLivingChildren);
                                    }

                                    iProb = infos().utils().modify(iProb, getTraitBirthModifier());
                                    iProb = infos().utils().modify(iProb, pLoopSpouse.getTraitBirthModifier());

                                    if (game().testRandomPercent(iProb))
                                    {
                                        if (isMale())
                                        {
                                            game().giveBirth(pLoopSpouse, this);
                                        }
                                        else
                                        {
                                            game().giveBirth(this, pLoopSpouse);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            // Dale END

            if ((game().getTurn() - getBirthTurn()) == infos().Globals.ADULT_AGE)
            {
                for (TraitType eLoopTrait = 0; eLoopTrait < infos().traitsNum(); eLoopTrait++)
                {
                    if (!isTrait(eLoopTrait))
                    {
                        if (game().randomNext(100) < infos().trait(eLoopTrait).miAdultProb)
                        {
                            addTrait(eLoopTrait);
                        }
                    }
                }
            }

            if ((game().getTurn() - getBirthTurn()) == (infos().Globals.ADULT_AGE + 1))
            {
                if (hasPlayer())
                {
                    if (!(player().doEventTrigger(infos().Globals.ADULTHOOD_EVENTTRIGGER, this)))
                    {
                        if (isLeaderOrSpouseOrHeirOrHeirSpouse())
                        {
                            player().pushLogData(TextManager.TEXT("TEXT_GAME_CHARACTER_ADULT_POPUP", HelpText.buildCharacterLinkVariable(this, true, player()), TEXTVAR(getAge())), GameLogType.CHARACTER_ADULT, getID());
                        }
                    }
                }
            }

            if (!isLeaderOrSuccessor())
            {
                if ((game().getTurn() - getBirthTurn()) == (infos().Globals.ADULT_AGE + 1))
                {
                    if (!hasArchetype())
                    {
                        generateArchetype(true);
                    }
                }
                else if ((game().getTurn() - getBirthTurn()) == (infos().Globals.ADULT_AGE + 2))
                {
                    if (!hasAdjective())
                    {
                        generateAdjective(true, true, true);
                    }
                }
            }

            if (game().randomNext(5) == 0)
            {
                assignNickname();
            }

            if (isTutor())
            {
                if (game().randomNext(100) < infos().Globals.TUTOR_PROB)
                {
                    Character pBestCharacter = null;
                    int iBestValue = 0;

                    using (var charListScoped = CollectionCache.GetListScoped<int>())
                    {
                        player().getActiveCharacters(charListScoped.Value);

                        foreach (int iLoopCharacter in charListScoped.Value)
                        {
                            Character pLoopCharacter = game().character(iLoopCharacter);

                            if (pLoopCharacter.canBeTutored())
                            {
                                int iValue = game().randomNext(1000) + 1;

                                if (pLoopCharacter.isLeader())
                                {
                                    iValue += 400;
                                }
                                else if (pLoopCharacter.isHeir())
                                {
                                    iValue += 200;
                                }
                                else if (pLoopCharacter.isLeaderChild())
                                {
                                    iValue += 100;
                                }

                                if (iValue > iBestValue)
                                {
                                    pBestCharacter = pLoopCharacter;
                                    iBestValue = iValue;
                                }
                            }
                        }
                    }

                    if (pBestCharacter != null)
                    {
                        if (!(player().doEventTrigger(infos().Globals.TUTOR_EVENTTRIGGER, this, pBestCharacter)))
                        {
                            RatingType eBestRating = (RatingType)(infos().utils().randomDieList(getRatingArray(), game().random()));

                            if (eBestRating != RatingType.NONE)
                            {
                                pBestCharacter.changeRating(eBestRating, 1);

                                using (var varListScope = CollectionCache.GetListScoped<TextVariable>())
                                using (var scope = CollectionCache.GetStringBuilderScoped())
                                {
                                    varListScope.Value.Add(HelpText.buildCharacterLinkVariable(this));
                                    varListScope.Value.Add(HelpText.buildTutorsLinkVariable(TEXTVAR("TEXT_GAME_CHARACTER_TUTORS_CHILD_VERB", TextManager)));
                                    varListScope.Value.Add(HelpText.buildCharacterLinkVariable(pBestCharacter, true, player()));
                                    varListScope.Value.Add(HelpText.buildRatingLinkVariable(eBestRating));
                                    player().addTurnSummary(TextManager.TEXT(scope.Value, "TEXT_GAME_CHARACTER_TUTORS_CHILD", varListScope.Value).ProfiledToString());
                                }
                            }
                        }
                    }
                }
            }

            if (isRoyal() && !isLeader() && isAdult())
            {
                if (game().randomNext(20) == 0)
                {
                    RatingType eRating = (RatingType)(game().randomNext((int)(infos().ratingsNum())));

                    int iChange = 0;

                    if (getRating(eRating) > 0)
                    {
                        iChange++;
                    }
                    else if (getRating(eRating) < 0)
                    {
                        iChange--;
                    }
                    else if (game().randomNext(2) == 0)
                    {
                        iChange++;
                    }
                    else
                    {
                        iChange--;
                    }

                    changeRating(eRating, iChange);

                    player().addTurnSummary(TextManager.TEXT("TEXT_GAME_CHARACTER_RATING_CHANGE", HelpText.buildCharacterLinkVariable(this), HelpText.buildSignedTextVariable(iChange), HelpText.buildRatingLinkVariable(eRating)));
                }
            }

            if (hasPlayer())
            {
                if (hasDesiredAmbition())
                {
                    if (!(player().isGoalEligible(getDesiredAmbition())) || (player().getNumAmbitions() >= (infos().Globals.MAX_AMBITIONS - 1)))
                    {
                        setDesiredAmbition(GoalType.NONE);
                    }
                }
                else if ((player().getNumAmbitions() > 0) && (player().getNumAmbitions() < (infos().Globals.MAX_AMBITIONS - 1)))
                {
                    if (isSuccessor() && isAdult() && (game().randomNext(100) == 0))
                    {
                        GoalType eBestGoal = GoalType.NONE;
                        int iBestValue = 0;

                        int iNextTier = (player().getNumAmbitions() + 2);

                        for (GoalType eLoopGoal = 0; eLoopGoal < infos().goalsNum(); eLoopGoal++)
                        {
                            if (player().isGoalTierValid(eLoopGoal, 2))
                            {
                                int iValue = (game().randomNext(1000) + 1);

                                foreach (TraitType eLoopTrait in getTraits())
                                {
                                    iValue += infos().goal(eLoopGoal).maiDesiredWeight[(int)eLoopTrait];
                                }

                                if (iValue > iBestValue)
                                {
                                    if (!(player().hasAmbition(eLoopGoal)))
                                    {
                                        if (player().isGoalEligible(eLoopGoal))
                                        {
                                            eBestGoal = eLoopGoal;
                                            iBestValue = iValue;
                                        }
                                    }
                                }
                            }
                        }

                        if (eBestGoal != GoalType.NONE)
                        {
                            setDesiredAmbition(eBestGoal);

                            player().addTurnSummary(TextManager.TEXT("TEXT_GAME_CHARACTER_DESIRED_AMBITION", HelpText.buildCharacterLinkVariable(this, true, player()), HelpText.buildGoalLinkVariable(eBestGoal, null, game(), false, getPlayer(), PlayerType.NONE, BarbarianType.NONE)));
                        }
                    }
                }
            }

            if (hasPlayer() && !(player().hasCharacterMissionAny(this)))
            {
                if (isRoyal() && !isRetired())
                {
                    if (canMarry() && isFertileAge())
                    {
                        if (isLeaderOrDescendant())
                        {
                            if (game().getTurn() > ((isMale()) ? 4 : 2))
                            {
                                if ((game().getTurn() % 3) == (getID() % 3))
                                {
                                    if (isHuman())
                                    {
                                        player().doEventTrigger(infos().Globals.MARRIAGE_OFFER_EVENTTRIGGER, this);
                                    }
                                    else
                                    {
                                        doMarriage(true);
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (game().randomNext(5) == 0)
                            {
                                doMarriage(false);
                            }
                        }
                    }
                }
            }

            // Dale - START
            if (hasPlayer() && !(player().hasCharacterMissionAny(this)))
            {
                if (!isRoyal() && !isRetired())
                {
                    if (canMarry() && isFertileAge())
                    {
                        if (game().randomNext(5) == 0)
                        {
                            doMarriage(false);
                            Debug.Log((object)"Character: " + this.getFullName(true, null, false, false, false, false, false) + " - got married");
                        }
                    }
                }
            }
            // Dale - END

            if (hasPlayer())
            {
                if (player().canDoEvents())
                {
                    for (EventClassType eLoopClass = 0; eLoopClass < infos().eventClassesNum(); eLoopClass++)
                    {
                        int iProb = infos().eventClass(eLoopClass).miCharacterProb;
                        if (iProb > 0)
                        {
                            if ((game().randomNext(100) < iProb) || game().TestEventStories)
                            {
                                player().doEventTrigger(EventTriggerType.NONE, this, null, -1, false, ulong.MaxValue, eLoopClass);
                            }
                        }
                    }

                    player().doEventTrigger(infos().Globals.NEW_TURN_CHARACTER_EVENTTRIGGER, this);
                }
                else
                {
                    if (isSuccessor())
                    {
                        if (!isAdult() && (getAge() > 10))
                        {
                            if (game().randomNext(2) == 0)
                            {
                                changeRating((RatingType)(game().randomNext((int)(infos().ratingsNum()))), 1);
                            }
                        }
                    }
                }
            }
        }
    }
}

            // Dale - Start
/*            else
            {
                if (!isRoyal() && !isRetired() && !isBarbarian())
                {
                    if (canMarry() && isFertileAge())
                    {
                        if (game().randomNext(5) == 0)
                        {
                            Debug.Log((object)"Character " + this.getFullName(true, null, false, false, false, false, false) + " - doMarriage");
                            doMarriage(false);
                        }
                    }
                }
            }

            if (isMale() && !(isRoyal() || isRoyalSpouse()))
            {
                if (canHaveChildren())
                {
                    foreach (int iSpouseID in getSpouses())
                    {
                        Character pLoopSpouse = game().character(iSpouseID);

                        if (pLoopSpouse.isAlive())
                        {
                            if (pLoopSpouse.canHaveChildren())
                            {
                                bool bValid = true;

                                if (isRetired() || pLoopSpouse.isRetired())
                                {
                                    bValid = false;
                                }

                                if (bValid)
                                {
                                    int iProb = 0;

                                    int iLivingChildren = countLivingChildren();

                                    if (iLivingChildren == 0)
                                    {
                                        iProb += 40;
                                    }
                                    else if (iLivingChildren == 1)
                                    {
                                        iProb += 30;
                                    }
                                    else
                                    {
                                        iProb += (20 / iLivingChildren);
                                    }

                                    iProb = infos().utils().modify(iProb, getTraitBirthModifier());
                                    iProb = infos().utils().modify(iProb, pLoopSpouse.getTraitBirthModifier());

                                    if (game().testRandomPercent(iProb))
                                    {
                                        Debug.Log((object)"Character " + this.getFullName(true, null, false, false, false, false, false) + " - giveBirth");
                                        if (isMale())
                                        {
                                            game().giveBirth(pLoopSpouse, this);
                                        }
                                        else
                                        {
                                            game().giveBirth(this, pLoopSpouse);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }*/
            // Dale - END
